(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('bluebird'), require('lodash'), require('elasticsearch'), require('azure-functions-ts-essentials')) :
	typeof define === 'function' && define.amd ? define(['exports', 'bluebird', 'lodash', 'elasticsearch', 'azure-functions-ts-essentials'], factory) :
	(factory((global.azureFunctionsElasticizer = global.azureFunctionsElasticizer || {}),global.bluebird,global._,global.elasticsearch,global._azureFunctionsTsEssentials));
}(this, (function (exports,bluebird,_,elasticsearch,azureFunctionsTsEssentials) { 'use strict';

var __assign = (undefined && undefined.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
var ELASTICSEARCH_REFRESH = 'wait_for';
var ELASTICSEARCH_MAX_RESULTS = 2147483519;
var getElasticClient = function (host) { return new elasticsearch.Client({
    host: host,
    defer: function () { return bluebird.Promise.defer(); }
}); };
var getStringDate = function (now) {
    if (!now)
        now = new Date();
    var year = now.getUTCFullYear();
    var month = (now.getUTCMonth() + 1).toString();
    var day = now.getUTCDate().toString();
    month = "" + (Number(month) < 10 ? '0' : '') + month;
    day = "" + (Number(day) < 10 ? '0' : '') + day;
    return year + "." + month + "." + day;
};
var getErrorResponse = function (err) {
    if (Array.isArray(err) && err.length > 0)
        return {
            status: azureFunctionsTsEssentials.HttpStatusCode.BadRequest,
            body: err.map(function (cur) { return ({
                type: _.get(cur, 'type'),
                message: _.get(cur, 'reason'),
                index: _.get(cur, 'index'),
                uuid: _.get(cur, 'index_uuid')
            }); })
        };
    return {
        status: err.status,
        body: {
            type: err.displayName,
            message: err.message
        }
    };
};
var Elasticizer = (function () {
    function Elasticizer(host, type, options) {
        this.host = host;
        this.type = type;
        this.options = options;
        this.client = getElasticClient(host);
        this.refresh = _.get(options, 'refresh', ELASTICSEARCH_REFRESH);
        this.prefix = _.get(options, 'prefix', '');
    }
    Elasticizer.prototype.getOne = function (index, id) {
        var query$ = this.client.get({
            index: "" + this.prefix + index,
            type: this.type,
            id: id
        });
        return query$
            .then(function (res) { return ({
            status: azureFunctionsTsEssentials.HttpStatusCode.OK,
            body: __assign({ _index: res._index, _id: res._id }, res._source)
        }); })
            .catch(getErrorResponse);
    };
    Elasticizer.prototype.search = function (index, body, query, page, perPage, sortAsc) {
        var _this = this;
        var query$ = this.client.search({
            index: Array.isArray(index)
                ? index.map(function (cur) { return "" + _this.prefix + cur; })
                : "" + this.prefix + index,
            body: body,
            q: query,
            from: Number(page) >= 0 && Number(perPage) > 0 ? Number(page) * Number(perPage) : 0,
            size: Number(perPage) > 0 ? Number(perPage) : ELASTICSEARCH_MAX_RESULTS,
            sort: "createdAtUtc:" + (sortAsc ? 'asc' : 'desc')
        });
        return query$
            .then(function (res) { return ({
            status: azureFunctionsTsEssentials.HttpStatusCode.OK,
            body: {
                data: res.hits.hits.map(function (cur) { return (__assign({ _index: cur._index, _id: cur._id }, cur._source)); }),
                hasMore: Number(page) >= 0 && Number(perPage) > 0
                    ? res.hits.total > (Number(page) + 1) * Number(perPage)
                    : false,
                totalCount: res.hits.total
            }
        }); })
            .catch(getErrorResponse);
    };
    Elasticizer.prototype.insertMany = function (req) {
        var _this = this;
        var contentType = req.headers ? req.headers['content-type'] : undefined;
        if (!(contentType && contentType.indexOf('application/json') >= 0))
            return Promise.resolve({
                status: azureFunctionsTsEssentials.HttpStatusCode.BadRequest,
                body: {
                    type: azureFunctionsTsEssentials.ErrorType.Invalid
                }
            });
        if (!(req.body && Object.keys(req.body).length))
            return Promise.resolve({
                status: azureFunctionsTsEssentials.HttpStatusCode.BadRequest,
                body: {
                    type: azureFunctionsTsEssentials.ErrorType.Invalid
                }
            });
        var query$ = this.client.bulk({
            type: this.type,
            refresh: this.refresh,
            body: req.body.reduce(function (acc, cur) {
                var index = { _index: "" + _this.prefix + cur._index };
                delete cur._index;
                acc.push({ index: __assign({}, index) }, __assign({}, cur, { createdAtUtc: new Date().toISOString() }));
                return acc;
            }, [])
        });
        return query$
            .then(function (res) {
            var isError = _.get(res, 'errors');
            if (isError)
                return getErrorResponse(res.items.map(function (cur) { return cur.index.error; }));
            return {
                status: azureFunctionsTsEssentials.HttpStatusCode.Created,
                body: {}
            };
        })
            .catch(getErrorResponse);
    };
    Elasticizer.prototype.updateOne = function (index, req, id) {
        var contentType = req.headers ? req.headers['content-type'] : undefined;
        if (!(contentType && contentType.indexOf('application/json') >= 0))
            return Promise.resolve({
                status: azureFunctionsTsEssentials.HttpStatusCode.BadRequest,
                body: {
                    type: azureFunctionsTsEssentials.ErrorType.Invalid
                }
            });
        if (!(req.body && Object.keys(req.body).length))
            return Promise.resolve({
                status: azureFunctionsTsEssentials.HttpStatusCode.BadRequest,
                body: {
                    type: azureFunctionsTsEssentials.ErrorType.Invalid
                }
            });
        if (!id)
            return Promise.resolve({
                status: azureFunctionsTsEssentials.HttpStatusCode.BadRequest,
                body: {
                    type: azureFunctionsTsEssentials.ErrorType.Invalid
                }
            });
        delete req.body._index;
        var query$ = this.client.update({
            index: "" + this.prefix + index,
            type: this.type,
            refresh: this.refresh,
            id: id,
            body: {
                doc: req.body
            }
        });
        return query$
            .then(function () { return ({
            status: azureFunctionsTsEssentials.HttpStatusCode.OK,
            body: {}
        }); })
            .catch(getErrorResponse);
    };
    Elasticizer.prototype.deleteOne = function (index, id) {
        if (!id)
            return Promise.resolve({
                status: azureFunctionsTsEssentials.HttpStatusCode.BadRequest,
                body: {
                    type: azureFunctionsTsEssentials.ErrorType.Missing
                }
            });
        var query$ = this.client.delete({
            index: "" + this.prefix + index,
            type: this.type,
            refresh: this.refresh,
            id: id
        });
        return query$
            .then(function () { return ({
            status: azureFunctionsTsEssentials.HttpStatusCode.OK,
            body: {}
        }); })
            .catch(getErrorResponse);
    };
    return Elasticizer;
}());

exports.ELASTICSEARCH_REFRESH = ELASTICSEARCH_REFRESH;
exports.ELASTICSEARCH_MAX_RESULTS = ELASTICSEARCH_MAX_RESULTS;
exports.getElasticClient = getElasticClient;
exports.getStringDate = getStringDate;
exports.Elasticizer = Elasticizer;

Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=azure-functions-elasticizer.umd.js.map
